@@ -411,3 +411,189 @@ class ManagedADComputer(ManagedADObject):
         self.kvno += 1
         logger.debug('Updated kvno for computer %s from %s to %s', self.computer_name, self.kvno - 1, self.kvno)
         self.set_password_locally(password)
+
+
+class ManagedADUser(ManagedADObject):
+
+    def __init__(self, samaccount_name: str, domain: 'ADDomain', location: str = None,
+                 password: str = None, encryption_types: List[ADEncryptionType] = None, kvno: int = None,
+                 common_name: str = None):
+        super().__init__(samaccount_name, domain, location, password)
+        self.common_name = common_name if common_name else self.samaccount_name
+        self.name = self.common_name
+        self.encryption_types = []
+        encryption_types = encryption_types if encryption_types else []
+        for enc_type in encryption_types:
+            original = enc_type
+            if isinstance(enc_type, str):
+                enc_type = ENCRYPTION_TYPE_STR_TO_ENUM.get(enc_type.lower())
+            if not isinstance(enc_type, ADEncryptionType):
+                raise ValueError('All specified encryption types must be ADEncryptionType enums or must '
+                                 'be strings convertible to ADEncryptionType enums. {} is neither.'
+                                 .format(original))
+            self.encryption_types.append(enc_type)
+        # assume the account is a new account unless given a kvno
+        self.kvno = 1 if kvno is None else kvno
+
+        self.kerberos_keys = []
+        self.raw_kerberos_keys = []
+        if self.password:
+            logger.debug('Generating kerberos keys from password during instantiation of user with name %s',
+                         self.name)
+            for enc_type in self.encryption_types:
+                raw_key = ad_password_string_to_key(enc_type, self.samaccount_name,
+                                                    self.password, self.domain_dns_name, is_user=True)
+                self.raw_kerberos_keys.append(raw_key)
+                # generate our user kerberos key
+                user_gss_kerberos_key = GssKerberosKey(self.samaccount_name, self.realm, raw_key, self.kvno,
+                                                       gss_name_type=AD_DEFAULT_NAME_TYPE)
+                self.kerberos_keys.append(user_gss_kerberos_key)
+            logger.debug('Generated %s kerberos keys from password during instantiation of user with name %s',
+                         len(self.kerberos_keys), self.name)
+
+    def add_encryption_type_locally(self, encryption_type: ADEncryptionType):
+        """ Adds an encryption type to the user locally. This will generate new kerberos keys
+        for the user using the new encryption type.
+        This function does nothing if the encryption type is already on the user.
+        This function raises an exception if the user's password is not set, as the password is
+        needed to generate new kerberos keys.
+        :param encryption_type: The encryption type to add to the user.
+        """
+        if encryption_type in self.encryption_types:
+            logger.debug(
+                'No change resulted from adding encryption type %s to user %s locally as it was already present',
+                encryption_type, self.name)
+            return
+        if self.password is None:
+            raise InvalidComputerParameterException('Encryption types can only be added to a user locally if its '
+                                                    'password is known. Without the password, new kerberos keys cannot '
+                                                    'be generated.')
+        logger.debug('Adding encryption type %s to user %s locally',
+                     encryption_type, self.name)
+        self.encryption_types.append(encryption_type)
+        raw_krb_key = ad_password_string_to_key(encryption_type, self.samaccount_name,
+                                                self.password, self.domain_dns_name, is_user=True)
+        self.raw_kerberos_keys.append(raw_krb_key)
+        user_gss_kerberos_key = GssKerberosKey(self.samaccount_name, self.realm, raw_krb_key, self.kvno,
+                                               gss_name_type=AD_DEFAULT_NAME_TYPE)
+        self.kerberos_keys.append(user_gss_kerberos_key)
+
+    def get_full_keytab_file_bytes_for_user(self) -> bytes:
+        """ Get the raw bytes that would comprise a complete keytab file for this user. The
+        resultant bytes form a file that can be used to initiate GSS security contexts as the user
+        with the user's principal name being the sAMAccountName.
+        """
+        return write_gss_kerberos_key_list_to_raw_bytes(self.kerberos_keys)
+
+    def get_computer_distinguished_name(self) -> str:
+        """ Get the LDAP distinguished name for the user. This raises an exception if location is not
+        set for the user.
+        """
+        if self.location is None:
+            raise InvalidComputerParameterException('The location of the computer is unknown and so a distinguished '
+                                                    'name cannot be determined for it.')
+        return construct_object_distinguished_name(self.common_name, self.location, self.domain_dns_name)
+
+    def get_encryption_types(self) -> List[ADEncryptionType]:
+        return self.encryption_types
+
+    def get_name(self) -> str:
+        return self.name
+
+    def get_user_kerberos_keys(self) -> List[GssKerberosKey]:
+        return self.kerberos_keys
+
+    def get_user_principal_name(self) -> str:
+        """ Gets the user principal name for the computer, to be used in initiating GSS security contexts """
+        return '{sam}@{realm}'.format(sam=self.samaccount_name, realm=self.realm)
+
+    def set_encryption_types_locally(self, encryption_types: List[ADEncryptionType]):
+        """ Sets the encryption types of the user locally. This will generate new kerberos keys
+        for the user using the new encryption types.
+        This function raises an exception if the user's password is not set, as the password is
+        needed to generate new kerberos keys.
+        :param encryption_types: The list of AD encryption types to set on the user.
+        """
+        if self.password is None:
+            raise InvalidComputerParameterException('Encryption types can only be set on a computer locally if its '
+                                                    'password is known. Without the password, new kerberos keys cannot '
+                                                    'be generated.')
+
+        new_kerberos_keys = []
+        new_raw_kerberos_keys = []
+        logger.debug('Adding new encryption types %s to user %s locally',
+                     encryption_types, self.name)
+        for encryption_type in encryption_types:
+            raw_krb_key = ad_password_string_to_key(encryption_type, self.samaccount_name,
+                                                    self.password, self.domain_dns_name, is_user=True)
+            new_raw_kerberos_keys.append(raw_krb_key)
+            user_gss_kerberos_key = GssKerberosKey(self.samaccount_name, self.realm, raw_krb_key, self.kvno,
+                                                   gss_name_type=AD_DEFAULT_NAME_TYPE)
+            new_kerberos_keys.append(user_gss_kerberos_key)
+        self.encryption_types = encryption_types
+        self.kerberos_keys = new_kerberos_keys
+        self.raw_kerberos_keys = new_raw_kerberos_keys
+        logger.debug('Generated %s new kerberos keys for new encryption types %s set on user %s locally',
+                     len(new_kerberos_keys), encryption_types, self.name)
+
+    def set_password_locally(self, password: str):
+        """ Sets the password on the AD user locally. This will regenerate user kerberos keys for all of the
+        encryption types on the user.
+        This function is meant to be used when the password was not set locally or was incorrectly set.
+        This function WILL NOT update the key version number of the kerberos keys; if a user's
+        password is actually changed, then update_password_locally should be used as that will update
+        the key version number properly and ensure the resultant kerberos keys can be properly used
+        for initiating and accepting security contexts.
+        :param password: The string password to set for the computer.
+        """
+        self.password = password
+        self.kerberos_keys = []
+        self.raw_kerberos_keys = []
+        logger.debug('Generating new kerberos keys for user %s based on new password',
+                     self.name)
+        for enc_type in self.encryption_types:
+            raw_key = ad_password_string_to_key(enc_type, self.samaccount_name,
+                                                self.password, self.domain_dns_name, is_user=True)
+            self.raw_kerberos_keys.append(raw_key)
+            # generate our user kerberos key
+            user_gss_kerberos_key = GssKerberosKey(self.samaccount_name, self.realm, raw_key, self.kvno,
+                                                   gss_name_type=AD_DEFAULT_NAME_TYPE)
+            self.kerberos_keys.append(user_gss_kerberos_key)
+        logger.info('Generated %s new kerberos keys for user %s based on new password and forgot old keys',
+                    len(self.kerberos_keys), self.name)
+
+    def write_keytab_file_for_user(self, file_path: str, merge_with_existing_file: bool = True):
+        """ Write all of the keytabs for this user to a file, which are the keys used to authenticate
+        with servers when acting as a client.
+
+        :param file_path: The path to the file where the keytabs will be written. If it does not exist, it will be
+                          created.
+        :param merge_with_existing_file: If True, the users' keytabs will be added into the keytab file at
+                                         `file_path` if one exists. If False, the file at `file_path` will be
+                                         overwritten if it exists. If the file does not exist, this does nothing.
+                                         Defaults to True.
+        """
+        logger.debug('Writing user key file for %s to %s', self.name, file_path)
+        entries_to_write = self.kerberos_keys
+        if merge_with_existing_file:
+            logger.debug('Merging with existing keytab file')
+            current_entries = process_keytab_file_to_extract_entries(file_path, must_exist=False)
+            logger.debug('%s existing keytabs found in file %s for merge', len(current_entries), file_path)
+            entries_to_write += current_entries
+        data = write_gss_kerberos_key_list_to_raw_bytes(entries_to_write)
+        self._write_keytab_data(file_path, data)
+        logger.info('Successfully wrote user key file with %s keys for %s to %s',
+                    len(self.kerberos_keys), self.name, file_path)
+
+    def _write_keytab_data(self, file_path: str, data: bytes):
+        with open(file_path, 'wb') as fp:
+            fp.write(data)
+
+    def update_password_locally(self, password: str):
+        """ Update the password for the computer locally and generate new kerberos keys for the new
+        password.
+        :param password: The string password to set for the computer.
+        """
+        self.kvno += 1
+        logger.debug('Updated kvno for user %s from %s to %s', self.name, self.kvno - 1, self.kvno)
+        self.set_password_locally(password)
